@@ -48,8 +48,8 @@
  *
  * As such, the data type conversion from gdk selection targets to OLE2 CF_* data
  * type specifiers is partially hardwired. Fixing this is complicated by (a) the
- * fact that the widget's declared selection types aren't accessible in calls here
- * that need to declare the corresponding OLE2 data types, and (b) there isn't a
+ * fact that the widget’s declared selection types aren’t accessible in calls here
+ * that need to declare the corresponding OLE2 data types, and (b) there isn’t a
  * 1-1 correspondence between gdk target types and OLE2 types. The former needs
  * some redesign in gtk dnd (something a gdk/gtk expert should do; I have tried
  * and failed!). As an example of the latter: gdk STRING, TEXT, COMPOUND_TEXT map
@@ -62,13 +62,17 @@
  * Note that in combination with another hack originated by Archaeopteryx
  * Software, the text conversions here may go to utf-8 unicode as the standard
  * within-gtk target or to single-byte ascii when the USE_ACP_TEXT compilation
- * flag is TRUE. This mode was added to support applications that aren't using
+ * flag is TRUE. This mode was added to support applications that aren’t using
  * utf-8 across the gtk/gdk API but instead use single-byte ascii according to
  * the current Windows code page. See gdkim-win32.c for more info on that.
  *
  */
- 
+
+/* The mingw.org compiler does not export GUIDS in it's import library. To work
+ * around that, define INITGUID to have the GUIDS declared. */
+#if defined(__MINGW32__) && !defined(__MINGW64_VERSION_MAJOR)
 #define INITGUID
+#endif
 
 #include "gdkdnd.h"
 #include "gdkproperty.h"
@@ -371,7 +375,7 @@ get_suggested_action (DWORD grfKeyState)
 }
 
 /* Process pending events -- we don't want to service non-GUI events
- * forever so do one iteration and then do more only if there's a
+ * forever so do one iteration and then do more only if there’s a
  * pending GDK event.
  */
 static void
@@ -821,7 +825,6 @@ idataobject_getdata (LPDATAOBJECT This,
 		     LPSTGMEDIUM  pMedium)
 {
   data_object *ctx = (data_object *) This;
-  GdkAtom target;
   HRESULT hr;
   GdkEvent e;
 
@@ -840,8 +843,6 @@ idataobject_getdata (LPDATAOBJECT This,
   active_pFormatEtc = pFormatEtc;
   active_pMedium = pMedium;
 
-  target = GDK_TARGET_STRING;
-
   e.type = GDK_SELECTION_REQUEST;
   e.selection.window = ctx->context->source_window;
   e.selection.send_event = FALSE; /* ??? */
@@ -1568,6 +1569,8 @@ add_format (GArray *fmts,
 void
 _gdk_dnd_init (void)
 {
+  CoInitializeEx (NULL, COINIT_APARTMENTTHREADED);
+
   if (getenv ("GDK_WIN32_USE_EXPERIMENTAL_OLE2_DND"))
     use_ole2_dnd = TRUE;
 
@@ -1614,6 +1617,8 @@ _gdk_win32_dnd_exit (void)
     {
       OleUninitialize ();
     }
+
+  CoUninitialize ();
 }
 
 /* Source side */
