@@ -30,10 +30,12 @@
 
 #include "gtkaccellabel.h"
 #include "gtkaccelmap.h"
-#include "gtkmain.h"
-#include "gtksizerequest.h"
-#include "gtkprivate.h"
 #include "gtkintl.h"
+#include "gtkmain.h"
+#include "gtkprivate.h"
+#include "gtkrender.h"
+#include "gtksizerequest.h"
+#include "gtkstylecontextprivate.h"
 
 /**
  * SECTION:gtkaccellabel
@@ -42,7 +44,7 @@
  * @See_also: #GtkAccelGroup
  *
  * The #GtkAccelLabel widget is a subclass of #GtkLabel that also displays an
- * accelerator key on the right of the label text, e.g. 'Ctl+S'.
+ * accelerator key on the right of the label text, e.g. “Ctl+S”.
  * It is commonly used in menus to show the keyboard short-cuts for commands.
  *
  * The accelerator key to display is not set explicitly.
@@ -51,10 +53,10 @@
  * gtk_accel_label_set_accel_widget().
  *
  * For example, a #GtkMenuItem widget may have an accelerator added to emit the
- * "activate" signal when the 'Ctl+S' key combination is pressed.
+ * “activate” signal when the “Ctl+S” key combination is pressed.
  * A #GtkAccelLabel is created and added to the #GtkMenuItem, and
  * gtk_accel_label_set_accel_widget() is called with the #GtkMenuItem as the
- * second argument. The #GtkAccelLabel will now display 'Ctl+S' after its label.
+ * second argument. The #GtkAccelLabel will now display “Ctl+S” after its label.
  *
  * Note that creating a #GtkMenuItem with gtk_menu_item_new_with_label() (or
  * one of the similar functions for #GtkCheckMenuItem and #GtkRadioMenuItem)
@@ -65,35 +67,37 @@
  * set (see #GtkAccelFlags).
  * A #GtkAccelLabel can display multiple accelerators and even signal names,
  * though it is almost always used to display just one accelerator key.
- * <example>
- * <title>Creating a simple menu item with an accelerator key.</title>
- * <programlisting>
+ *
+ * ## Creating a simple menu item with an accelerator key.
+ *
+ * |[<!-- language="C" -->
  *   GtkWidget *save_item;
  *   GtkAccelGroup *accel_group;
  *
- *   /<!---->* Create a GtkAccelGroup and add it to the window. *<!---->/
- *   accel_group = gtk_accel_group_new (<!-- -->);
+ *   // Create a GtkAccelGroup and add it to the window.
+ *   accel_group = gtk_accel_group_new ();
  *   gtk_window_add_accel_group (GTK_WINDOW (window), accel_group);
  *
- *   /<!---->* Create the menu item using the convenience function. *<!---->/
+ *   // Create the menu item using the convenience function.
  *   save_item = gtk_menu_item_new_with_label ("Save");
  *   gtk_widget_show (save_item);
  *   gtk_container_add (GTK_CONTAINER (menu), save_item);
  *
- *   /<!---->* Now add the accelerator to the GtkMenuItem. Note that since we called
- *      gtk_menu_item_new_with_label(<!-- -->) to create the GtkMenuItem the
- *      GtkAccelLabel is automatically set up to display the GtkMenuItem
- *      accelerators. We just need to make sure we use GTK_ACCEL_VISIBLE here. *<!---->/
+ *   // Now add the accelerator to the GtkMenuItem. Note that since we
+ *   // called gtk_menu_item_new_with_label() to create the GtkMenuItem
+ *   // the GtkAccelLabel is automatically set up to display the
+ *   // GtkMenuItem accelerators. We just need to make sure we use
+ *   // GTK_ACCEL_VISIBLE here.
  *   gtk_widget_add_accelerator (save_item, "activate", accel_group,
  *                               GDK_KEY_s, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
- * </programlisting>
- * </example>
+ * ]|
  */
 
 enum {
   PROP_0,
   PROP_ACCEL_CLOSURE,
-  PROP_ACCEL_WIDGET
+  PROP_ACCEL_WIDGET,
+  LAST_PROP
 };
 
 struct _GtkAccelLabelPrivate
@@ -109,6 +113,8 @@ struct _GtkAccelLabelPrivate
   GdkModifierType accel_mods;
 };
 
+GParamSpec *props[LAST_PROP] = { NULL, };
+
 static void         gtk_accel_label_set_property (GObject            *object,
 						  guint               prop_id,
 						  const GValue       *value,
@@ -129,7 +135,7 @@ static void         gtk_accel_label_get_preferred_width (GtkWidget           *wi
                                                          gint                *nat_width);
 
 
-G_DEFINE_TYPE (GtkAccelLabel, gtk_accel_label, GTK_TYPE_LABEL)
+G_DEFINE_TYPE_WITH_PRIVATE (GtkAccelLabel, gtk_accel_label, GTK_TYPE_LABEL)
 
 static void
 gtk_accel_label_class_init (GtkAccelLabelClass *class)
@@ -182,22 +188,21 @@ gtk_accel_label_class_init (GtkAccelLabelClass *class)
 
 #endif /* GDK_WINDOWING_QUARTZ */
 
-  g_object_class_install_property (gobject_class,
-                                   PROP_ACCEL_CLOSURE,
-                                   g_param_spec_boxed ("accel-closure",
-						       P_("Accelerator Closure"),
-						       P_("The closure to be monitored for accelerator changes"),
-						       G_TYPE_CLOSURE,
-						       GTK_PARAM_READWRITE));
-  g_object_class_install_property (gobject_class,
-                                   PROP_ACCEL_WIDGET,
-                                   g_param_spec_object ("accel-widget",
-                                                        P_("Accelerator Widget"),
-                                                        P_("The widget to be monitored for accelerator changes"),
-                                                        GTK_TYPE_WIDGET,
-                                                        GTK_PARAM_READWRITE));
+  props[PROP_ACCEL_CLOSURE] =
+    g_param_spec_boxed ("accel-closure",
+                        P_("Accelerator Closure"),
+                        P_("The closure to be monitored for accelerator changes"),
+                        G_TYPE_CLOSURE,
+                        GTK_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY);
 
-  g_type_class_add_private (gobject_class, sizeof (GtkAccelLabelPrivate));
+  props[PROP_ACCEL_WIDGET] =
+    g_param_spec_object ("accel-widget",
+                         P_("Accelerator Widget"),
+                         P_("The widget to be monitored for accelerator changes"),
+                         GTK_TYPE_WIDGET,
+                         GTK_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY);
+
+  g_object_class_install_properties (gobject_class, LAST_PROP, props);
 }
 
 static void
@@ -253,9 +258,7 @@ gtk_accel_label_init (GtkAccelLabel *accel_label)
 {
   GtkAccelLabelPrivate *priv;
 
-  accel_label->priv = G_TYPE_INSTANCE_GET_PRIVATE (accel_label,
-                                                   GTK_TYPE_ACCEL_LABEL,
-                                                   GtkAccelLabelPrivate);
+  accel_label->priv = gtk_accel_label_get_instance_private (accel_label);
   priv = accel_label->priv;
 
   priv->accel_padding = 3;
@@ -344,10 +347,43 @@ gtk_accel_label_get_accel_width (GtkAccelLabel *accel_label)
 	  (accel_label->priv->accel_string_width ? accel_label->priv->accel_padding : 0));
 }
 
+static PangoLayout *
+gtk_accel_label_get_accel_layout (GtkAccelLabel *accel_label)
+{
+  GtkWidget *widget = GTK_WIDGET (accel_label);
+  GtkStyleContext *context;
+  PangoAttrList *attrs;
+  PangoLayout *layout;
+  PangoFontDescription *font_desc;
+
+  context = gtk_widget_get_style_context (widget);
+
+  gtk_style_context_save (context);
+  gtk_style_context_add_class (context, GTK_STYLE_CLASS_ACCELERATOR);
+
+  layout = gtk_widget_create_pango_layout (widget, gtk_accel_label_get_string (accel_label));
+
+  attrs = _gtk_style_context_get_pango_attributes (context);
+  if (!attrs)
+    attrs = pango_attr_list_new ();
+  gtk_style_context_get (context,
+                         gtk_widget_get_state_flags (widget),
+                         "font", &font_desc,
+                         NULL);
+  pango_attr_list_change (attrs, pango_attr_font_desc_new (font_desc));
+  pango_font_description_free (font_desc);
+  pango_layout_set_attributes (layout, attrs);
+  pango_attr_list_unref (attrs);
+
+  gtk_style_context_restore (context);
+
+  return layout;
+}
+
 static void
-gtk_accel_label_get_preferred_width (GtkWidget       *widget,
-                                     gint            *min_width,
-                                     gint            *nat_width)
+gtk_accel_label_get_preferred_width (GtkWidget *widget,
+                                     gint      *min_width,
+                                     gint      *nat_width)
 {
   GtkAccelLabel *accel_label = GTK_ACCEL_LABEL (widget);
   PangoLayout   *layout;
@@ -355,8 +391,7 @@ gtk_accel_label_get_preferred_width (GtkWidget       *widget,
 
   GTK_WIDGET_CLASS (gtk_accel_label_parent_class)->get_preferred_width (widget, min_width, nat_width);
 
-  layout = gtk_widget_create_pango_layout (GTK_WIDGET (widget), 
-					   gtk_accel_label_get_string (accel_label));
+  layout = gtk_accel_label_get_accel_layout (accel_label);
   pango_layout_get_pixel_size (layout, &width, NULL);
   accel_label->priv->accel_string_width = width;
 
@@ -381,7 +416,6 @@ gtk_accel_label_draw (GtkWidget *widget,
                       cairo_t   *cr)
 {
   GtkAccelLabel *accel_label = GTK_ACCEL_LABEL (widget);
-  GtkMisc *misc = GTK_MISC (accel_label);
   GtkTextDirection direction;
   guint ac_width;
   GtkAllocation allocation;
@@ -401,7 +435,6 @@ gtk_accel_label_draw (GtkWidget *widget,
 
       gint x;
       gint y;
-      gint xpad;
 
       context = gtk_widget_get_style_context (widget);
       label_layout = gtk_label_get_layout (GTK_LABEL (accel_label));
@@ -409,7 +442,8 @@ gtk_accel_label_draw (GtkWidget *widget,
       cairo_save (cr);
 
       /* XXX: Mad hack: We modify the label's width so it renders
-       * properly in its draw function that we chain to. */
+       * properly in its draw function that we chain to.
+       */
       if (direction == GTK_TEXT_DIR_RTL)
         cairo_translate (cr, ac_width, 0);
       if (gtk_label_get_ellipsize (label))
@@ -419,34 +453,28 @@ gtk_accel_label_draw (GtkWidget *widget,
       
       allocation.width -= ac_width;
       gtk_widget_set_allocation (widget, &allocation);
-      if (GTK_WIDGET_CLASS (gtk_accel_label_parent_class)->draw)
-        GTK_WIDGET_CLASS (gtk_accel_label_parent_class)->draw (widget,
-                                                               cr);
+      GTK_WIDGET_CLASS (gtk_accel_label_parent_class)->draw (widget, cr);
       allocation.width += ac_width;
       gtk_widget_set_allocation (widget, &allocation);
       if (gtk_label_get_ellipsize (label))
         pango_layout_set_width (label_layout,
-                                pango_layout_get_width (label_layout) 
+                                pango_layout_get_width (label_layout)
                                 + ac_width * PANGO_SCALE);
 
       cairo_restore (cr);
 
-      gtk_misc_get_padding (misc, &xpad, NULL);
-
       if (direction == GTK_TEXT_DIR_RTL)
-        x = xpad;
+        x = 0;
       else
-        x = gtk_widget_get_allocated_width (widget) - xpad - ac_width;
+        x = gtk_widget_get_allocated_width (widget) - ac_width;
 
       gtk_label_get_layout_offsets (GTK_LABEL (accel_label), NULL, &y);
 
-      accel_layout = gtk_widget_create_pango_layout (widget, gtk_accel_label_get_string (accel_label));
-
+      accel_layout = gtk_accel_label_get_accel_layout (accel_label);
       y += get_first_baseline (label_layout) - get_first_baseline (accel_layout) - allocation.y;
 
       gtk_style_context_save (context);
       gtk_style_context_add_class (context, GTK_STYLE_CLASS_ACCELERATOR);
-
       gtk_render_layout (context, cr, x, y, accel_layout);
       gtk_style_context_restore (context);
 
@@ -454,8 +482,7 @@ gtk_accel_label_draw (GtkWidget *widget,
     }
   else
     {
-      if (GTK_WIDGET_CLASS (gtk_accel_label_parent_class)->draw)
-        GTK_WIDGET_CLASS (gtk_accel_label_parent_class)->draw (widget, cr);
+      GTK_WIDGET_CLASS (gtk_accel_label_parent_class)->draw (widget, cr);
     }
   
   return FALSE;
@@ -492,7 +519,7 @@ accel_widget_weak_ref_cb (GtkAccelLabel *accel_label,
                                         refetch_widget_accel_closure,
                                         accel_label);
   accel_label->priv->accel_widget = NULL;
-  g_object_notify (G_OBJECT (accel_label), "accel-widget");
+  g_object_notify_by_pspec (G_OBJECT (accel_label), props[PROP_ACCEL_WIDGET]);
 }
 
 /**
@@ -531,18 +558,14 @@ gtk_accel_label_set_accel_widget (GtkAccelLabel *accel_label,
                                    accel_label, G_CONNECT_SWAPPED);
           refetch_widget_accel_closure (accel_label);
         }
-      g_object_notify (G_OBJECT (accel_label), "accel-widget");
+      g_object_notify_by_pspec (G_OBJECT (accel_label), props[PROP_ACCEL_WIDGET]);
     }
 }
 
 static void
 gtk_accel_label_reset (GtkAccelLabel *accel_label)
 {
-  if (accel_label->priv->accel_string)
-    {
-      g_free (accel_label->priv->accel_string);
-      accel_label->priv->accel_string = NULL;
-    }
+  g_clear_pointer (&accel_label->priv->accel_string, g_free);
   
   gtk_widget_queue_resize (GTK_WIDGET (accel_label));
 }
@@ -594,7 +617,7 @@ gtk_accel_label_set_accel_closure (GtkAccelLabel *accel_label,
 				   accel_label, 0);
 	}
       gtk_accel_label_reset (accel_label);
-      g_object_notify (G_OBJECT (accel_label), "accel-closure");
+      g_object_notify_by_pspec (G_OBJECT (accel_label), props[PROP_ACCEL_CLOSURE]);
     }
 }
 
@@ -616,7 +639,7 @@ gtk_accel_label_get_string (GtkAccelLabel *accel_label)
 }
 
 /* Underscores in key names are better displayed as spaces
- * E.g., Page_Up should be "Page Up".
+ * E.g., Page_Up should be “Page Up”.
  *
  * Some keynames also have prefixes that are not suitable
  * for display, e.g XF86AudioMute, so strip those out, too.
@@ -836,12 +859,13 @@ _gtk_accel_label_class_get_accelerator_label (GtkAccelLabelClass *klass,
 #endif
       seen_mod = TRUE;
     }
-  if (seen_mod)
-    g_string_append (gstring, klass->mod_separator);
   
   ch = gdk_keyval_to_unicode (accelerator_key);
   if (ch && ch < 0x80 && (g_unichar_isgraph (ch) || ch == ' '))
     {
+      if (seen_mod)
+        g_string_append (gstring, klass->mod_separator);
+
       switch (ch)
 	{
 	case ' ':
@@ -862,6 +886,9 @@ _gtk_accel_label_class_get_accelerator_label (GtkAccelLabelClass *klass,
       tmp = gdk_keyval_name (gdk_keyval_to_lower (accelerator_key));
       if (tmp != NULL)
 	{
+          if (seen_mod)
+            g_string_append (gstring, klass->mod_separator);
+
 	  if (tmp[0] != 0 && tmp[1] == 0)
 	    g_string_append_c (gstring, g_ascii_toupper (tmp[0]));
 	  else
@@ -896,11 +923,7 @@ gtk_accel_label_refetch (GtkAccelLabel *accel_label)
 
   g_return_val_if_fail (GTK_IS_ACCEL_LABEL (accel_label), FALSE);
 
-  if (accel_label->priv->accel_string)
-    {
-      g_free (accel_label->priv->accel_string);
-      accel_label->priv->accel_string = NULL;
-    }
+  g_clear_pointer (&accel_label->priv->accel_string, g_free);
 
   g_object_get (gtk_widget_get_settings (GTK_WIDGET (accel_label)),
                 "gtk-enable-accels", &enable_accels,
@@ -939,12 +962,10 @@ gtk_accel_label_refetch (GtkAccelLabel *accel_label)
       if (have_accel)
 	{
 	  GtkAccelLabelClass *klass;
-	  gchar *tmp;
 
 	  klass = GTK_ACCEL_LABEL_GET_CLASS (accel_label);
-	  tmp = _gtk_accel_label_class_get_accelerator_label (klass, accel_key, accel_mods);
-	  accel_label->priv->accel_string = g_strconcat ("   ", tmp, NULL);
-	  g_free (tmp);
+	  accel_label->priv->accel_string =
+	      _gtk_accel_label_class_get_accelerator_label (klass, accel_key, accel_mods);
 	}
 
       else
@@ -981,8 +1002,32 @@ gtk_accel_label_set_accel (GtkAccelLabel   *accel_label,
                            guint            accelerator_key,
                            GdkModifierType  accelerator_mods)
 {
+  g_return_if_fail (GTK_IS_ACCEL_LABEL (accel_label));
+
   accel_label->priv->accel_key = accelerator_key;
   accel_label->priv->accel_mods = accelerator_mods;
 
   gtk_accel_label_reset (accel_label);
 }
+
+/**
+ * gtk_accel_label_get_accel:
+ * @accel_label: a #GtkAccelLabel
+ * @accelerator_key: (out): return location for the keyval
+ * @accelerator_mods: (out): return location for the modifier mask
+ *
+ * Gets the keyval and modifier mask set with
+ * gtk_accel_label_set_accel().
+ *
+ * Since: 3.12
+ */
+void
+gtk_accel_label_get_accel (GtkAccelLabel   *accel_label,
+                           guint           *accelerator_key,
+                           GdkModifierType *accelerator_mods)
+{
+  g_return_if_fail (GTK_IS_ACCEL_LABEL (accel_label));
+
+  *accelerator_key = accel_label->priv->accel_key;
+  *accelerator_mods = accel_label->priv->accel_mods;
+}
